@@ -39,6 +39,8 @@ FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );
 #include <pic8259.h>
 #include <ipxe/malloc.h>
 #include <ipxe/device.h>
+#include <ipxe/timer.h>
+#include <ipxe/quiesce.h>
 #include <ipxe/cpuid.h>
 #include <ipxe/msr.h>
 #include <ipxe/hyperv.h>
@@ -51,6 +53,12 @@ FILE_LICENCE ( GPL2_OR_LATER_OR_UBDL );
  */
 #define HV_MESSAGE_MAX_WAIT_MS 1000
 
+/** Hyper-V timer frequency (fixed 10Mhz) */
+#define HV_TIMER_HZ 10000000
+
+/** Hyper-V timer scale factor (used to avoid 64-bit division) */
+#define HV_TIMER_SHIFT 18
+
 /**
  * Convert a Hyper-V status code to an iPXE status code
  *
@@ -145,36 +153,48 @@ static void hv_free_message ( struct hv_hypervisor *hv ) {
 /**
  * Check whether or not we are running in Hyper-V
  *
- * @v hv		Hyper-V hypervisor
  * @ret rc		Return status code
  */
-static int hv_check_hv ( struct hv_hypervisor *hv ) {
+static int hv_check_hv ( void ) {
 	struct x86_features features;
 	uint32_t interface_id;
 	uint32_t discard_ebx;
 	uint32_t discard_ecx;
 	uint32_t discard_edx;
-	uint32_t available;
-	uint32_t permissions;
 
 	/* Check for presence of a hypervisor (not necessarily Hyper-V) */
 	x86_features ( &features );
 	if ( ! ( features.intel.ecx & CPUID_FEATURES_INTEL_ECX_HYPERVISOR ) ) {
-		DBGC ( hv, "HV %p not running in a hypervisor\n", hv );
+		DBGC ( HV_INTERFACE_ID, "HV not running in a hypervisor\n" );
 		return -ENODEV;
 	}
 
 	/* Check that hypervisor is Hyper-V */
-	cpuid ( HV_CPUID_INTERFACE_ID, &interface_id, &discard_ebx,
+	cpuid ( HV_CPUID_INTERFACE_ID, 0, &interface_id, &discard_ebx,
 		&discard_ecx, &discard_edx );
 	if ( interface_id != HV_INTERFACE_ID ) {
-		DBGC ( hv, "HV %p not running in Hyper-V (interface ID "
-		       "%#08x)\n", hv, interface_id );
+		DBGC ( HV_INTERFACE_ID, "HV not running in Hyper-V (interface "
+		       "ID %#08x)\n", interface_id );
 		return -ENODEV;
 	}
 
+	return 0;
+}
+
+/**
+ * Check required features
+ *
+ * @v hv		Hyper-V hypervisor
+ * @ret rc		Return status code
+ */
+static int hv_check_features ( struct hv_hypervisor *hv ) {
+	uint32_t available;
+	uint32_t permissions;
+	uint32_t discard_ecx;
+	uint32_t discard_edx;
+
 	/* Check that required features and privileges are available */
-	cpuid ( HV_CPUID_FEATURES, &available, &permissions, &discard_ecx,
+	cpuid ( HV_CPUID_FEATURES, 0, &available, &permissions, &discard_ecx,
 		&discard_edx );
 	if ( ! ( available & HV_FEATURES_AVAIL_HYPERCALL_MSR ) ) {
 		DBGC ( hv, "HV %p has no hypercall MSRs (features %08x:%08x)\n",
@@ -204,9 +224,8 @@ static int hv_check_hv ( struct hv_hypervisor *hv ) {
  * Map hypercall page
  *
  * @v hv		Hyper-V hypervisor
- * @ret rc		Return status code
  */
-static int hv_map_hypercall ( struct hv_hypervisor *hv ) {
+static void hv_map_hypercall ( struct hv_hypervisor *hv ) {
 	union {
 		struct {
 			uint32_t ebx;
@@ -226,19 +245,18 @@ static int hv_map_hypercall ( struct hv_hypervisor *hv ) {
 	/* Report guest OS identity */
 	guest_os_id = rdmsr ( HV_X64_MSR_GUEST_OS_ID );
 	if ( guest_os_id != 0 ) {
-		DBGC ( hv, "HV %p guest OS ID MSR already set to %#08llx\n",
+		DBGC ( hv, "HV %p guest OS ID MSR was %#08llx\n",
 		       hv, guest_os_id );
-		return -EBUSY;
 	}
 	guest_os_id = HV_GUEST_OS_ID_IPXE;
 	DBGC2 ( hv, "HV %p guest OS ID MSR is %#08llx\n", hv, guest_os_id );
 	wrmsr ( HV_X64_MSR_GUEST_OS_ID, guest_os_id );
 
 	/* Get hypervisor system identity (for debugging) */
-	cpuid ( HV_CPUID_VENDOR_ID, &discard_eax, &vendor_id.ebx,
+	cpuid ( HV_CPUID_VENDOR_ID, 0, &discard_eax, &vendor_id.ebx,
 		&vendor_id.ecx, &vendor_id.edx );
 	vendor_id.text[ sizeof ( vendor_id.text ) - 1 ] = '\0';
-	cpuid ( HV_CPUID_HYPERVISOR_ID, &build, &version, &discard_ecx,
+	cpuid ( HV_CPUID_HYPERVISOR_ID, 0, &build, &version, &discard_ecx,
 		&discard_edx );
 	DBGC ( hv, "HV %p detected \"%s\" version %d.%d build %d\n", hv,
 	       vendor_id.text, ( version >> 16 ), ( version & 0xffff ), build );
@@ -249,8 +267,6 @@ static int hv_map_hypercall ( struct hv_hypervisor *hv ) {
 	hypercall |= ( virt_to_phys ( hv->hypercall ) | HV_HYPERCALL_ENABLE );
 	DBGC2 ( hv, "HV %p hypercall MSR is %#08llx\n", hv, hypercall );
 	wrmsr ( HV_X64_MSR_HYPERCALL, hypercall );
-
-	return 0;
 }
 
 /**
@@ -278,13 +294,16 @@ static void hv_unmap_hypercall ( struct hv_hypervisor *hv ) {
  * Map synthetic interrupt controller
  *
  * @v hv		Hyper-V hypervisor
- * @ret rc		Return status code
  */
-static int hv_map_synic ( struct hv_hypervisor *hv ) {
+static void hv_map_synic ( struct hv_hypervisor *hv ) {
 	uint64_t simp;
 	uint64_t siefp;
 	uint64_t scontrol;
 
+	/* Zero SynIC message and event pages */
+	memset ( hv->synic.message, 0, PAGE_SIZE );
+	memset ( hv->synic.event, 0, PAGE_SIZE );
+
 	/* Map SynIC message page */
 	simp = rdmsr ( HV_X64_MSR_SIMP );
 	simp &= ( PAGE_SIZE - 1 );
@@ -304,26 +323,17 @@ static int hv_map_synic ( struct hv_hypervisor *hv ) {
 	scontrol |= HV_SCONTROL_ENABLE;
 	DBGC2 ( hv, "HV %p SCONTROL MSR is %#08llx\n", hv, scontrol );
 	wrmsr ( HV_X64_MSR_SCONTROL, scontrol );
-
-	return 0;
 }
 
 /**
- * Unmap synthetic interrupt controller
+ * Unmap synthetic interrupt controller, leaving SCONTROL untouched
  *
  * @v hv		Hyper-V hypervisor
  */
-static void hv_unmap_synic ( struct hv_hypervisor *hv ) {
-	uint64_t scontrol;
+static void hv_unmap_synic_no_scontrol ( struct hv_hypervisor *hv ) {
 	uint64_t siefp;
 	uint64_t simp;
 
-	/* Disable SynIC */
-	scontrol = rdmsr ( HV_X64_MSR_SCONTROL );
-	scontrol &= ~HV_SCONTROL_ENABLE;
-	DBGC2 ( hv, "HV %p SCONTROL MSR is %#08llx\n", hv, scontrol );
-	wrmsr ( HV_X64_MSR_SCONTROL, scontrol );
-
 	/* Unmap SynIC event page */
 	siefp = rdmsr ( HV_X64_MSR_SIEFP );
 	siefp &= ( ( PAGE_SIZE - 1 ) & ~HV_SIEFP_ENABLE );
@@ -337,6 +347,24 @@ static void hv_unmap_synic ( struct hv_hypervisor *hv ) {
 	wrmsr ( HV_X64_MSR_SIMP, simp );
 }
 
+/**
+ * Unmap synthetic interrupt controller
+ *
+ * @v hv		Hyper-V hypervisor
+ */
+static void hv_unmap_synic ( struct hv_hypervisor *hv ) {
+	uint64_t scontrol;
+
+	/* Disable SynIC */
+	scontrol = rdmsr ( HV_X64_MSR_SCONTROL );
+	scontrol &= ~HV_SCONTROL_ENABLE;
+	DBGC2 ( hv, "HV %p SCONTROL MSR is %#08llx\n", hv, scontrol );
+	wrmsr ( HV_X64_MSR_SCONTROL, scontrol );
+
+	/* Unmap SynIC event and message pages */
+	hv_unmap_synic_no_scontrol ( hv );
+}
+
 /**
  * Enable synthetic interrupt
  *
@@ -373,8 +401,12 @@ void hv_disable_sint ( struct hv_hypervisor *hv, unsigned int sintx ) {
 	unsigned long msr = HV_X64_MSR_SINT ( sintx );
 	uint64_t sint;
 
-	/* Disable synthetic interrupt */
+	/* Do nothing if interrupt is already disabled */
 	sint = rdmsr ( msr );
+	if ( sint & HV_SINT_MASKED )
+		return;
+
+	/* Disable synthetic interrupt */
 	sint &= ~HV_SINT_AUTO_EOI;
 	sint |= HV_SINT_MASKED;
 	DBGC2 ( hv, "HV %p SINT%d MSR is %#08llx\n", hv, sintx, sint );
@@ -509,6 +541,10 @@ static int hv_probe ( struct root_device *rootdev ) {
 	struct hv_hypervisor *hv;
 	int rc;
 
+	/* Check we are running in Hyper-V */
+	if ( ( rc = hv_check_hv() ) != 0 )
+		goto err_check_hv;
+
 	/* Allocate and initialise structure */
 	hv = zalloc ( sizeof ( *hv ) );
 	if ( ! hv ) {
@@ -516,9 +552,9 @@ static int hv_probe ( struct root_device *rootdev ) {
 		goto err_alloc;
 	}
 
-	/* Check we are running in Hyper-V */
-	if ( ( rc = hv_check_hv ( hv ) ) != 0 )
-		goto err_check_hv;
+	/* Check features */
+	if ( ( rc = hv_check_features ( hv ) ) != 0 )
+		goto err_check_features;
 
 	/* Allocate pages */
 	if ( ( rc = hv_alloc_pages ( hv, &hv->hypercall, &hv->synic.message,
@@ -530,12 +566,10 @@ static int hv_probe ( struct root_device *rootdev ) {
 		goto err_alloc_message;
 
 	/* Map hypercall page */
-	if ( ( rc = hv_map_hypercall ( hv ) ) != 0 )
-		goto err_map_hypercall;
+	hv_map_hypercall ( hv );
 
 	/* Map synthetic interrupt controller */
-	if ( ( rc = hv_map_synic ( hv ) ) != 0 )
-		goto err_map_synic;
+	hv_map_synic ( hv );
 
 	/* Probe Hyper-V devices */
 	if ( ( rc = vmbus_probe ( hv, &rootdev->dev ) ) != 0 )
@@ -547,17 +581,16 @@ static int hv_probe ( struct root_device *rootdev ) {
 	vmbus_remove ( hv, &rootdev->dev );
  err_vmbus_probe:
 	hv_unmap_synic ( hv );
- err_map_synic:
 	hv_unmap_hypercall ( hv );
- err_map_hypercall:
 	hv_free_message ( hv );
  err_alloc_message:
 	hv_free_pages ( hv, hv->hypercall, hv->synic.message, hv->synic.event,
 			NULL );
  err_alloc_pages:
- err_check_hv:
+ err_check_features:
 	free ( hv );
  err_alloc:
+ err_check_hv:
 	return rc;
 }
 
@@ -576,6 +609,7 @@ static void hv_remove ( struct root_device *rootdev ) {
 	hv_free_pages ( hv, hv->hypercall, hv->synic.message, hv->synic.event,
 			NULL );
 	free ( hv );
+	rootdev_set_drvdata ( rootdev, NULL );
 }
 
 /** Hyper-V root device driver */
@@ -590,6 +624,167 @@ struct root_device hv_root_device __root_device = {
 	.driver = &hv_root_driver,
 };
 
+/**
+ * Quiesce system
+ *
+ */
+static void hv_quiesce ( void ) {
+	struct hv_hypervisor *hv = rootdev_get_drvdata ( &hv_root_device );
+	unsigned int i;
+
+	/* Do nothing if we are not running in Hyper-V */
+	if ( ! hv )
+		return;
+
+	/* The "enlightened" portions of the Windows Server 2016 boot
+	 * process will not cleanly take ownership of an active
+	 * Hyper-V connection.  Experimentation shows that the minimum
+	 * requirement is that we disable the SynIC message page
+	 * (i.e. zero the SIMP MSR).
+	 *
+	 * We cannot perform a full shutdown of the Hyper-V
+	 * connection.  Experimentation shows that if we disable the
+	 * SynIC (i.e. zero the SCONTROL MSR) then Windows Server 2016
+	 * will enter an indefinite wait loop.
+	 *
+	 * Attempt to create a safe handover environment by resetting
+	 * all MSRs except for SCONTROL.
+	 *
+	 * Note that we do not shut down our VMBus devices, since we
+	 * may need to unquiesce the system and continue operation.
+	 */
+
+	/* Disable all synthetic interrupts */
+	for ( i = 0 ; i <= HV_SINT_MAX ; i++ )
+		hv_disable_sint ( hv, i );
+
+	/* Unmap synthetic interrupt controller, leaving SCONTROL
+	 * enabled (see above).
+	 */
+	hv_unmap_synic_no_scontrol ( hv );
+
+	/* Unmap hypercall page */
+	hv_unmap_hypercall ( hv );
+
+	DBGC ( hv, "HV %p quiesced\n", hv );
+}
+
+/**
+ * Unquiesce system
+ *
+ */
+static void hv_unquiesce ( void ) {
+	struct hv_hypervisor *hv = rootdev_get_drvdata ( &hv_root_device );
+	uint64_t simp;
+	int rc;
+
+	/* Do nothing if we are not running in Hyper-V */
+	if ( ! hv )
+		return;
+
+	/* Experimentation shows that the "enlightened" portions of
+	 * Windows Server 2016 will break our Hyper-V connection at
+	 * some point during a SAN boot.  Surprisingly it does not
+	 * change the guest OS ID MSR, but it does leave the SynIC
+	 * message page disabled.
+	 *
+	 * Our own explicit quiescing procedure will also disable the
+	 * SynIC message page.  We can therefore use the SynIC message
+	 * page enable bit as a heuristic to determine when we need to
+	 * reestablish our Hyper-V connection.
+	 */
+	simp = rdmsr ( HV_X64_MSR_SIMP );
+	if ( simp & HV_SIMP_ENABLE )
+		return;
+
+	/* Remap hypercall page */
+	hv_map_hypercall ( hv );
+
+	/* Remap synthetic interrupt controller */
+	hv_map_synic ( hv );
+
+	/* Reset Hyper-V devices */
+	if ( ( rc = vmbus_reset ( hv, &hv_root_device.dev ) ) != 0 ) {
+		DBGC ( hv, "HV %p could not unquiesce: %s\n",
+		       hv, strerror ( rc ) );
+		/* Nothing we can do */
+		return;
+	}
+}
+
+/** Hyper-V quiescer */
+struct quiescer hv_quiescer __quiescer = {
+	.quiesce = hv_quiesce,
+	.unquiesce = hv_unquiesce,
+};
+
+/**
+ * Probe timer
+ *
+ * @ret rc		Return status code
+ */
+static int hv_timer_probe ( void ) {
+	uint32_t available;
+	uint32_t discard_ebx;
+	uint32_t discard_ecx;
+	uint32_t discard_edx;
+	int rc;
+
+	/* Check we are running in Hyper-V */
+	if ( ( rc = hv_check_hv() ) != 0 )
+		return rc;
+
+	/* Check for available reference counter */
+	cpuid ( HV_CPUID_FEATURES, 0, &available, &discard_ebx, &discard_ecx,
+		&discard_edx );
+	if ( ! ( available & HV_FEATURES_AVAIL_TIME_REF_COUNT_MSR ) ) {
+		DBGC ( HV_INTERFACE_ID, "HV has no time reference counter\n" );
+		return -ENODEV;
+	}
+
+	return 0;
+}
+
+/**
+ * Get current system time in ticks
+ *
+ * @ret ticks		Current time, in ticks
+ */
+static unsigned long hv_currticks ( void ) {
+
+	/* Calculate time using a combination of bit shifts and
+	 * multiplication (to avoid a 64-bit division).
+	 */
+	return ( ( rdmsr ( HV_X64_MSR_TIME_REF_COUNT ) >> HV_TIMER_SHIFT ) *
+		 ( TICKS_PER_SEC / ( HV_TIMER_HZ >> HV_TIMER_SHIFT ) ) );
+}
+
+/**
+ * Delay for a fixed number of microseconds
+ *
+ * @v usecs		Number of microseconds for which to delay
+ */
+static void hv_udelay ( unsigned long usecs ) {
+	uint32_t start;
+	uint32_t elapsed;
+	uint32_t threshold;
+
+	/* Spin until specified number of 10MHz ticks have elapsed */
+	start = rdmsr ( HV_X64_MSR_TIME_REF_COUNT );
+	threshold = ( usecs * ( HV_TIMER_HZ / 1000000 ) );
+	do {
+		elapsed = ( rdmsr ( HV_X64_MSR_TIME_REF_COUNT ) - start );
+	} while ( elapsed < threshold );
+}
+
+/** Hyper-V timer */
+struct timer hv_timer __timer ( TIMER_PREFERRED ) = {
+	.name = "Hyper-V",
+	.probe = hv_timer_probe,
+	.currticks = hv_currticks,
+	.udelay = hv_udelay,
+};
+
 /* Drag in objects via hv_root_device */
 REQUIRING_SYMBOL ( hv_root_device );
 
